__revision__ = "/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Job.py 0.96.91.D001 2005/09/08 09:14:36 knight"
class Jobs:
    def __init__(self, num, taskmaster):
        self.job = None
        if num > 1:
            try:
                self.job = Parallel(taskmaster, num)
                self.num_jobs = num
            except NameError:
                pass
        if self.job is None:
            self.job = Serial(taskmaster)
            self.num_jobs = 1
    def run(self):
        try:
            self.job.start()
        except KeyboardInterrupt:
            import signal
            signal.signal(signal.SIGINT, signal.SIG_IGN)
            raise
class Serial:
    def __init__(self, taskmaster):
        self.taskmaster = taskmaster
    def start(self):
        while 1:
            task = self.taskmaster.next_task()
            if task is None: break
            try:
                task.prepare()
                task.execute()
            except KeyboardInterrupt:
                raise
            except:
                task.exception_set()
                task.failed()
            else:
                task.executed()
            task.postprocess()
try:
    import Queue
    import threading
except ImportError:
    pass
else:
    class Worker(threading.Thread):
        def __init__(self, requestQueue, resultsQueue):
            threading.Thread.__init__(self)
            self.setDaemon(1)
            self.requestQueue = requestQueue
            self.resultsQueue = resultsQueue
            self.start()

        def run(self):
            while 1:
                task = self.requestQueue.get()
                try:
                    task.execute()
                except KeyboardInterrupt:
                    ok = False
                except:
                    task.exception_set()
                    ok = 0
                else: ok = 1
                self.resultsQueue.put((task, ok))
    class ThreadPool:
        def __init__(self, num):
            self.requestQueue = Queue.Queue(0)
            self.resultsQueue = Queue.Queue(0)
            for i in range(num): Worker(self.requestQueue, self.resultsQueue)
        def put(self, obj):
            self.requestQueue.put(obj)
        def get(self, block = 1):
            return self.resultsQueue.get(block)
        def preparation_failed(self, obj):
            self.resultsQueue.put((obj, 0))
    class Parallel:
        def __init__(self, taskmaster, num):
            self.taskmaster = taskmaster
            self.tp = ThreadPool(num)
            self.maxjobs = num
        def start(self):
            jobs = 0
            while 1:
                while jobs < self.maxjobs:
                    task = self.taskmaster.next_task()
                    if task is None: break
                    try:
                        task.prepare()
                    except KeyboardInterrupt:
                        raise
                    except:
                        task.exception_set()
                        self.tp.preparation_failed(task)
                        jobs = jobs + 1
                        continue
                    self.tp.put(task)
                    jobs = jobs + 1
                if not task and not jobs: break
                while 1:
                    task, ok = self.tp.get()
                    jobs = jobs - 1
                    if ok: task.executed()
                    else: task.failed()
                    task.postprocess()
                    if self.tp.resultsQueue.empty(): break
