__revision__="/home/scons/scons/branch.0/branch.96/baseline/src/engine/SCons/Node/FS.py 0.96.91.D001 2005/09/08 09:14:36 knight"
import os
import os.path
import shutil
import stat
import string
import sys
import time
import cStringIO
import SCons.Action
from SCons.Debug import logInstanceCreation
import SCons.Errors
import SCons.Node
import SCons.Sig.MD5
import SCons.Util
import SCons.Warnings
default_max_drift=2*24*60*60
Save_Strings=None
def save_strings(val):
    global Save_Strings
    Save_Strings=val
def copy3(src, dest):
    if os.path.islink(src):
        os.symlink(src, dest)
    else:
        shutil.copy2(src, dest)
def _copy_func(src, dest):
    shutil.copy2(src, dest)
    st=os.stat(src)
    os.chmod(dest, stat.S_IMODE(st[stat.ST_MODE]) | stat.S_IWRITE)
Valid_Duplicates=['hard-soft-copy', 'soft-hard-copy', 'hard-copy', 'soft-copy', 'copy']
Link_Funcs=[]
def set_duplicate(duplicate):
    try:
        _hardlink_func=os.link
    except AttributeError:
        _hardlink_func=None
    try:
        _softlink_func=os.symlink
    except AttributeError:
        _softlink_func=None
    link_dict={
        'hard' : _hardlink_func,
        'soft' : _softlink_func,
        'copy' : _copy_func
    }
    if not duplicate in Valid_Duplicates:
        raise SCons.Errors.InternalError, ("The argument of set_duplicate should be in Valid_Duplicates")
    global Link_Funcs
    Link_Funcs=[]
    for func in string.split(duplicate,'-'):
        if link_dict[func]:
            Link_Funcs.append(link_dict[func])

def LinkFunc(target, source, env):
    src=source[0].abspath
    dest=target[0].abspath
    dir, file=os.path.split(dest)
    if dir and not target[0].fs.isdir(dir):
        os.makedirs(dir)
    if not Link_Funcs:
        set_duplicate('hard-soft-copy')
    for func in Link_Funcs:
        try:
            func(src,dest)
            break
        except (IOError, OSError):
            if func==Link_Funcs[-1]: raise
            else: pass
    return 0

Link=SCons.Action.Action(LinkFunc, None)
def LocalString(target, source, env):
    return 'Local copy of %s from %s' % (target[0], source[0])
LocalCopy=SCons.Action.Action(LinkFunc, LocalString)

def UnlinkFunc(target, source, env):
    t=target[0]
    t.fs.unlink(t.abspath)
    return 0
Unlink=SCons.Action.Action(UnlinkFunc, None)

def MkdirFunc(target, source, env):
    t=target[0]
    if not t.exists():
        t.fs.mkdir(t.abspath)
    return 0
Mkdir=SCons.Action.Action(MkdirFunc, None, presub=None)
MkdirBuilder=None

def get_MkdirBuilder():
    global MkdirBuilder
    if MkdirBuilder is None:
        import SCons.Builder
        MkdirBuilder=SCons.Builder.Builder(action=Mkdir, env=None, explain=None, is_explicit=None, name="MkdirBuilder")
    return MkdirBuilder

def CacheRetrieveFunc(target, source, env):
    t=target[0]
    fs=t.fs
    cachedir, cachefile=t.cachepath()
    if fs.exists(cachefile):
        if SCons.Action.execute_actions:
            fs.copy2(cachefile, t.path)
            st=fs.stat(cachefile)
            fs.chmod(t.path, stat.S_IMODE(st[stat.ST_MODE]) | stat.S_IWRITE)
        return 0
    return 1

def CacheRetrieveString(target, source, env):
    t=target[0]
    cachedir, cachefile=t.cachepath()
    if t.fs.exists(cachefile):
        return "Retrieved `%s' from cache" % t.path
    return None

CacheRetrieve=SCons.Action.Action(CacheRetrieveFunc, CacheRetrieveString)

CacheRetrieveSilent=SCons.Action.Action(CacheRetrieveFunc, None)

def CachePushFunc(target, source, env):
    t=target[0]
    fs=t.fs
    cachedir, cachefile=t.cachepath()
    if fs.exists(cachefile): return
    if not fs.isdir(cachedir): fs.makedirs(cachedir)

    tempfile=cachefile+'.tmp'
    try:
        fs.copy2(t.path, tempfile)
        fs.rename(tempfile, cachefile)
        st=fs.stat(t.path)
        fs.chmod(cachefile, stat.S_IMODE(st[stat.ST_MODE]) | stat.S_IWRITE)
    except (IOError, OSError):
        SCons.Warnings.warn(SCons.Warnings.CacheWriteErrorWarning, "Unable to copy %s to cache. Cache file is %s" % (str(target), cachefile))
        return

CachePush=SCons.Action.Action(CachePushFunc, None)

class _Null:
    pass

_null=_Null()
_is_cygwin=sys.platform=="cygwin"
if os.path.normcase("TeSt")==os.path.normpath("TeSt") and not _is_cygwin:
    def _my_normcase(x):
        return x
else:
    def _my_normcase(x):
        return string.upper(x)

class DiskChecker:
    def __init__(self, type, do, ignore):
        self.type=type
        self.do=do
        self.ignore=ignore
        self.set_do()
    def set_do(self):
        self.__call__=self.do
    def set_ignore(self):
        self.__call__=self.ignore
    def set(self, list):
        if self.type in list:
            self.set_do()
        else:
            self.set_ignore()

def do_diskcheck_match(node, predicate, errorfmt):
    path=node.abspath
    if predicate(path):
        raise TypeError, errorfmt % path

def ignore_diskcheck_match(node, predicate, errorfmt):
    pass

diskcheck_match=DiskChecker('match', do_diskcheck_match, ignore_diskcheck_match)

diskcheckers=[ diskcheck_match, ]
def set_diskcheck(list):
    for dc in diskcheckers: dc.set(list)
def diskcheck_types():
    return map(lambda dc: dc.type, diskcheckers)
class EntryProxy(SCons.Util.Proxy):
    def __get_abspath(self):
        entry=self.get()
        return SCons.Util.SpecialAttrWrapper(entry.get_abspath(), entry.name + "_abspath")
    def __get_filebase(self):
        name=self.get().name
        return SCons.Util.SpecialAttrWrapper(SCons.Util.splitext(name)[0], name + "_filebase")
    def __get_suffix(self):
        name=self.get().name
        return SCons.Util.SpecialAttrWrapper(SCons.Util.splitext(name)[1], name + "_suffix")
    def __get_file(self):
        name=self.get().name
        return SCons.Util.SpecialAttrWrapper(name, name + "_file")
    def __get_base_path(self):
        entry=self.get()
        return SCons.Util.SpecialAttrWrapper(SCons.Util.splitext(entry.get_path())[0], entry.name + "_base")
    def __get_posix_path(self):
        if os.sep=='/': return self
        else:
            entry=self.get()
            r=string.replace(entry.get_path(), os.sep, '/')
            return SCons.Util.SpecialAttrWrapper(r, entry.name + "_posix")
    def __get_win32_path(self):
        if os.sep=='\\':
            return self
        else:
            entry=self.get()
            r=string.replace(entry.get_path(), os.sep, '\\')
            return SCons.Util.SpecialAttrWrapper(r, entry.name + "_win32")

    def __get_srcnode(self):
        return EntryProxy(self.get().srcnode())

    def __get_srcdir(self):
        return EntryProxy(self.get().srcnode().dir)

    def __get_rsrcnode(self):
        return EntryProxy(self.get().srcnode().rfile())

    def __get_rsrcdir(self):
        return EntryProxy(self.get().srcnode().rfile().dir)

    def __get_dir(self):
        return EntryProxy(self.get().dir)
    
    dictSpecialAttrs={ "base"     : __get_base_path,
                         "posix"    : __get_posix_path,
                         "win32"    : __get_win32_path,
                         "srcpath"  : __get_srcnode,
                         "srcdir"   : __get_srcdir,
                         "dir"      : __get_dir,
                         "abspath"  : __get_abspath,
                         "filebase" : __get_filebase,
                         "suffix"   : __get_suffix,
                         "file"     : __get_file,
                         "rsrcpath" : __get_rsrcnode,
                         "rsrcdir"  : __get_rsrcdir,
                       }

    def __getattr__(self, name):
        try:
            return self.dictSpecialAttrs[name](self)
        except KeyError:
            try:
                attr=SCons.Util.Proxy.__getattr__(self, name)
            except AttributeError:
                entry=self.get()
                classname=string.split(str(entry.__class__), '.')[-1]
                if classname[-2:]=="'>":
                    classname=classname[:-2]
                raise AttributeError, "%s instance '%s' has no attribute '%s'" % (classname, entry.name, name)
            return attr

class Base(SCons.Node.Node):
    def __init__(self, name, directory, fs):
        if __debug__: logInstanceCreation(self, 'Node.FS.Base')
        SCons.Node.Node.__init__(self)

        self.name=name
        self.fs=fs

        assert directory, "A directory must be provided"

        self.abspath=directory.entry_abspath(name)
        if directory.path=='.':
            self.path=name
        else:
            self.path=directory.entry_path(name)
        if directory.tpath=='.':
            self.tpath=name
        else:
            self.tpath=directory.entry_tpath(name)
        self.path_elements=directory.path_elements + [self]

        self.dir=directory
        self.cwd=None
        self.duplicate=directory.duplicate

    def clear(self):
        """__cache_reset__"""
        SCons.Node.Node.clear(self)
    def get_dir(self):
        return self.dir
    def get_suffix(self):
        "__cacheable__"
        return SCons.Util.splitext(self.name)[1]
    def rfile(self):
        return self
    def __str__(self):
        global Save_Strings
        if Save_Strings: return self._save_str()
        return self._get_str()
    def _save_str(self):
        "__cacheable__"
        return self._get_str()
    def _get_str(self):
        if self.duplicate or self.is_derived(): return self.get_path()
        return self.srcnode().get_path()
    rstr=__str__
    def stat(self):
        "__cacheable__"
        try: return self.fs.stat(self.abspath)
        except os.error: return None
    def exists(self):
        "__cacheable__"
        return not self.stat() is None
    def rexists(self):
        "__cacheable__"
        return self.rfile().exists()
    def getmtime(self):
        st=self.stat()
        if st: return self.stat()[stat.ST_MTIME]
        else: return None
    def getsize(self):
        st=self.stat()
        if st: return self.stat()[stat.ST_SIZE]
        else: return None
    def isdir(self):
        st=self.stat()
        return not st is None and stat.S_ISDIR(st[stat.ST_MODE])
    def isfile(self):
        st=self.stat()
        return not st is None and stat.S_ISREG(st[stat.ST_MODE])
    if hasattr(os, 'symlink'):
        def islink(self):
            try: st=self.fs.lstat(self.abspath)
            except os.error: return 0
            return stat.S_ISLNK(st[stat.ST_MODE])
    else:
        def islink(self):
            return 0
    def is_under(self, dir):
        if self is dir: return 1
        else: return self.dir.is_under(dir)
    def set_local(self):
        self._local=1
    def srcnode(self):
        """__cacheable__"""
        dir=self.dir
        name=self.name
        while dir:
            if dir.srcdir:
                srcnode=self.fs.Entry(name, dir.srcdir,
                                        klass=self.__class__)
                return srcnode
            name=dir.name + os.sep + name
            dir=dir.up()
        return self
    def get_path(self, dir=None):
        if not dir:
            dir=self.fs.getcwd()
        if self==dir:
            return '.'
        path_elems=self.path_elements
        try: i=path_elems.index(dir)
        except ValueError: pass
        else: path_elems=path_elems[i+1:]
        path_elems=map(lambda n: n.name, path_elems)
        return string.join(path_elems, os.sep)

    def set_src_builder(self, builder):
        self.sbuilder=builder
        if not self.has_builder():
            self.builder_set(builder)

    def src_builder(self):
        try:
            scb=self.sbuilder
        except AttributeError:
            scb=self.dir.src_builder()
            self.sbuilder=scb
        return scb

    def get_abspath(self):
        return self.abspath

    def for_signature(self):
        return self.name

    def get_subst_proxy(self):
        try:
            return self._proxy
        except AttributeError:
            ret=EntryProxy(self)
            self._proxy=ret
            return ret

class Entry(Base):
    def diskcheck_match(self):
        pass

    def disambiguate(self):
        if self.isdir():
            self.__class__=Dir
            self._morph()
        else:
            self.__class__=File
            self._morph()
            self.clear()
        return self

    def rfile(self):
        self.__class__=File
        self._morph()
        self.clear()
        return File.rfile(self)

    def get_found_includes(self, env, scanner, path):
        return self.disambiguate().get_found_includes(env, scanner, path)

    def scanner_key(self):
        return self.get_suffix()

    def get_contents(self):
        if self.isfile():
            self.__class__=File
            self._morph()
            return self.get_contents()
        if self.isdir():
            self.__class__=Dir
            self._morph()
            return self.get_contents()
        if self.islink():
            return ''
        raise AttributeError
    def rel_path(self, other):
        return self.disambiguate().rel_path(other)
    def exists(self):
        return self.disambiguate().exists()
    def calc_signature(self, calc=None):
        return self.disambiguate().calc_signature(calc)
    def must_be_a_Dir(self):
        self.__class__=Dir
        self._morph()
        return self
_classEntry=Entry
class LocalFS:

    if SCons.Memoize.use_memoizer:
        __metaclass__=SCons.Memoize.Memoized_Metaclass
    def chmod(self, path, mode):
        return os.chmod(path, mode)
    def copy2(self, src, dst):
        return copy3(src, dst)
    def exists(self, path):
        return os.path.exists(path)
    def getmtime(self, path):
        return os.path.getmtime(path)
    def getsize(self, path):
        return os.path.getsize(path)
    def isdir(self, path):
        return os.path.isdir(path)
    def isfile(self, path):
        return os.path.isfile(path)
    def link(self, src, dst):
        return os.link(src, dst)
    def lstat(self, path):
        return os.lstat(path)
    def listdir(self, path):
        return os.listdir(path)
    def makedirs(self, path):
        return os.makedirs(path)
    def mkdir(self, path):
        return os.mkdir(path)
    def rename(self, old, new):
        return os.rename(old, new)
    def stat(self, path):
        return os.stat(path)
    def symlink(self, src, dst):
        return os.symlink(src, dst)
    def open(self, path):
        return open(path)
    def unlink(self, path):
        return os.unlink(path)

    if hasattr(os, 'symlink'):
        def islink(self, path):
            return os.path.islink(path)
    else:
        def islink(self, path):
            return 0

if SCons.Memoize.use_old_memoization():
    _FSBase=LocalFS
    class LocalFS(SCons.Memoize.Memoizer, _FSBase):
        def __init__(self, *args, **kw):
            apply(_FSBase.__init__, (self,)+args, kw)
            SCons.Memoize.Memoizer.__init__(self)

class FS(LocalFS):
    def __init__(self, path=None):
        if __debug__: logInstanceCreation(self, 'Node.FS')
        self.Root={}
        self.SConstruct_dir=None
        self.CachePath=None
        self.cache_force=None
        self.cache_show=None
        self.max_drift=default_max_drift
        if path is None: self.pathTop=os.getcwd()
        else: self.pathTop=path

        self.Top=self._doLookup(Dir, os.path.normpath(self.pathTop))
        self.Top.path='.'
        self.Top.tpath='.'
        self._cwd=self.Top

    def clear_cache(self):
        "__cache_reset__"
        pass
    
    def set_SConstruct_dir(self, dir):
        self.SConstruct_dir=dir

    def get_max_drift(self):
        return self.max_drift

    def set_max_drift(self, max_drift):
        self.max_drift=max_drift

    def getcwd(self):
        return self._cwd

    def __checkClass(self, node, klass):
        if isinstance(node, klass) or klass==Entry:
            return node
        if node.__class__==Entry:
            node.__class__=klass
            node._morph()
            return node
        raise TypeError, "Tried to lookup %s '%s' as a %s." % \
              (node.__class__.__name__, node.path, klass.__name__)
        
    def _doLookup(self, fsclass, name, directory=None, create=1):
        """__cacheable__"""
        if not name:
            name='.'
        path_orig=string.split(name, os.sep)
        path_norm=string.split(_my_normcase(name), os.sep)

        first_orig=path_orig.pop(0)
        first_norm=path_norm.pop(0)

        drive, path_first=os.path.splitdrive(first_orig)
        if path_first:
            path_orig=[ path_first, ] + path_orig
            path_norm=[ _my_normcase(path_first), ] + path_norm
        else:
            drive=_my_normcase(drive)
            try:
                directory=self.Root[drive]
            except KeyError:
                if not create:
                    raise SCons.Errors.UserError
                directory=RootDir(drive, self)
                self.Root[drive]=directory

        if not path_orig:
            return directory

        last_orig=path_orig.pop()
        last_norm=path_norm.pop()
            
        for orig, norm in map(None, path_orig, path_norm):
            try:
                entries=directory.entries
            except AttributeError:
                directory.must_be_a_Dir()
                entries=directory.entries
            try:
                directory=entries[norm]
            except KeyError:
                if not create: raise SCons.Errors.UserError
                d=Dir(orig, directory, self)
                d.diskcheck_match()
                directory.entries[norm]=d
                directory.add_wkid(d)
                directory=d
        directory.must_be_a_Dir()
        try:
            e=directory.entries[last_norm]
        except KeyError:
            if not create: raise SCons.Errors.UserError
            result=fsclass(last_orig, directory, self)
            result.diskcheck_match()
            directory.entries[last_norm]=result 
            directory.add_wkid(result)
        else:
            result=self.__checkClass(e, fsclass)
        return result 

    def _transformPath(self, name, directory):
        if name and name[0] =='#':
            directory=self.Top
            name=name[1:]
            if name and (name[0] ==os.sep or name[0] =='/'):
                name=name[1:]
            name=os.path.join('.', os.path.normpath(name))
        elif not directory:
            directory=self._cwd
        return (os.path.normpath(name), directory)

    def chdir(self, dir, change_os_dir=0):
        curr=self._cwd
        try:
            if not dir is None:
                self._cwd=dir
                if change_os_dir:
                    os.chdir(dir.abspath)
        except OSError:
            self._cwd=curr
            raise
    def Entry(self, name, directory=None, create=1, klass=None):
        if not klass:
            klass=Entry

        if isinstance(name, Base):
            return self.__checkClass(name, klass)
        else:
            if directory and not isinstance(directory, Dir):
                directory=self.Dir(directory)
            name, directory=self._transformPath(name, directory)
            return self._doLookup(klass, name, directory, create)
    
    def File(self, name, directory=None, create=1):
        return self.Entry(name, directory, create, File)
    
    def Dir(self, name, directory=None, create=1):
        return self.Entry(name, directory, create, Dir)
    
    def BuildDir(self, build_dir, src_dir, duplicate=1):
        if not isinstance(src_dir, SCons.Node.Node):
            src_dir=self.Dir(src_dir)
        if not isinstance(build_dir, SCons.Node.Node):
            build_dir=self.Dir(build_dir)
        if src_dir.is_under(build_dir):
            raise SCons.Errors.UserError, "Source directory cannot be under build directory."
        if build_dir.srcdir:
            if build_dir.srcdir ==src_dir: return
            raise SCons.Errors.UserError, "'%s' already has a source directory: '%s'."%(build_dir, build_dir.srcdir)
        build_dir.link(src_dir, duplicate)

    def Repository(self, *dirs):
        for d in dirs:
            if not isinstance(d, SCons.Node.Node):
                d=self.Dir(d)
            self.Top.addRepository(d)

    def Rfindalldirs(self, pathlist, cwd):
        """__cacheable__"""
        if SCons.Util.is_String(pathlist):
            pathlist=string.split(pathlist, os.pathsep)
        if not SCons.Util.is_List(pathlist):
            pathlist=[pathlist]
        result=[]
        for path in filter(None, pathlist):
            if isinstance(path, SCons.Node.Node):
                result.append(path)
                continue
            path, dir=self._transformPath(path, cwd)
            dir=dir.Dir(path)
            result.extend(dir.get_all_rdirs())
        return result

    def CacheDir(self, path):
        self.CachePath=path

    def build_dir_target_climb(self, orig, dir, tail):
        """__cacheable__"""
        targets=[]
        message=None
        fmt="building associated BuildDir targets: %s"
        start_dir=dir
        while dir:
            for bd in dir.build_dirs:
                if start_dir.is_under(bd):
                    return [orig], fmt % str(orig)
                p=apply(os.path.join, [bd.path] + tail)
                targets.append(self.Entry(p))
            tail=[dir.name] + tail
            dir=dir.up()
        if targets:
            message=fmt % string.join(map(str, targets))
        return targets, message

class Dir(Base):
    def __init__(self, name, directory, fs):
        if __debug__: logInstanceCreation(self, 'Node.FS.Dir')
        Base.__init__(self, name, directory, fs)
        self._morph()

    def _morph(self):
        """__cache_reset__"""
        self.repositories=[]
        self.srcdir=None
        self.entries={}
        self.entries['.']=self
        self.entries['..']=self.dir
        self.cwd=self
        self.searched=0
        self._sconsign=None
        self.build_dirs=[]
        self.builder=get_MkdirBuilder()
        self.get_executor().set_action_list(self.builder.action)
    def diskcheck_match(self):
        diskcheck_match(self, self.fs.isfile, "File %s found where directory expected.")
    def disambiguate(self):
        return self
    def __clearRepositoryCache(self, duplicate=None):
        for node in self.entries.values():
            if node !=self.dir:
                if node !=self and isinstance(node, Dir):
                    node.__clearRepositoryCache(duplicate)
                else:
                    node.clear()
                    try:
                        del node._srcreps
                    except AttributeError:
                        pass
                    if duplicate !=None:
                        node.duplicate=duplicate
    
    def __resetDuplicate(self, node):
        if node !=self:
            node.duplicate=node.get_dir().duplicate

    def Entry(self, name):
        return self.fs.Entry(name, self)

    def Dir(self, name):
        return self.fs.Dir(name, self)

    def File(self, name):
        return self.fs.File(name, self)

    def link(self, srcdir, duplicate):
        self.srcdir=srcdir
        self.duplicate=duplicate
        self.__clearRepositoryCache(duplicate)
        srcdir.build_dirs.append(self)

    def getRepositories(self):
        """__cacheable__"""
        if self.srcdir and not self.duplicate:
            return self.srcdir.get_all_rdirs() + self.repositories
        return self.repositories

    def get_all_rdirs(self):
        """__cacheable__"""
        result=[self]
        fname='.'
        dir=self
        while dir:
            for rep in dir.getRepositories():
                result.append(rep.Dir(fname))
            fname=dir.name + os.sep + fname
            dir=dir.up()
        return result

    def addRepository(self, dir):
        if dir !=self and not dir in self.repositories:
            self.repositories.append(dir)
            dir.tpath='.'
            self.__clearRepositoryCache()

    def up(self):
        return self.entries['..']

    def rel_path(self, other):
        """__cacheable__"""
        if isinstance(other, Dir):
            name=[]
        else:
            try:
                name=[other.name]
                other=other.dir
            except AttributeError:
                return str(other)
        if self is other:
            return name and name[0] or '.'
        i=0
        for x, y in map(None, self.path_elements, other.path_elements):
            if not x is y:
                break
            i=i + 1
        path_elems=['..']*(len(self.path_elements)-i) \
                   + map(lambda n: n.name, other.path_elements[i:]) \
                   + name
             
        return string.join(path_elems, os.sep)

    def scan(self):
        if not self.implicit is None:
            return
        self.implicit=[]
        self.implicit_dict={}
        self._children_reset()

        dont_scan=lambda k: k not in ['.', '..', '.sconsign']
        deps=filter(dont_scan, self.entries.keys())
        deps.sort()
        entries=map(lambda n, e=self.entries: e[n], deps)

        self._add_child(self.implicit, self.implicit_dict, entries)

    def get_found_includes(self, env, scanner, path):
        """__cacheable__"""
        if not scanner:
            return []
        self.clear()
        return scanner(self, env, path)

    def build(self, **kw):
        global MkdirBuilder
        if not self.builder is MkdirBuilder:
            apply(SCons.Node.Node.build, [self,], kw)

    def _create(self):
        listDirs=[]
        parent=self
        while parent:
            if parent.exists():
                break
            listDirs.append(parent)
            p=parent.up()
            if p is None:
                raise SCons.Errors.StopError, parent.path
            parent=p
        listDirs.reverse()
        for dirnode in listDirs:
            try:
                SCons.Node.Node.build(dirnode)
                dirnode.get_executor().nullify()
                dirnode.clear()
            except OSError:
                pass

    def multiple_side_effect_has_builder(self):
        global MkdirBuilder
        return not self.builder is MkdirBuilder and self.has_builder()

    def alter_targets(self):
        return self.fs.build_dir_target_climb(self, self, [])

    def scanner_key(self):
        return None

    def get_contents(self):
        contents=cStringIO.StringIO()
        for kid in self.children():
            contents.write(kid.get_contents())
        return contents.getvalue()

    def prepare(self):
        pass

    def do_duplicate(self, src):
        pass

    def current(self, calc=None):
        if not self.builder is MkdirBuilder and not self.exists():
            return 0
        state=0
        for kid in self.children():
            s=kid.get_state()
            if s and (not state or s > state):
                state=s
        import SCons.Node
        if state ==0 or state ==SCons.Node.up_to_date:
            return 1
        else:
            return 0

    def rdir(self):
        "__cacheable__"
        if not self.exists():
            norm_name=_my_normcase(self.name)
            for dir in self.dir.get_all_rdirs():
                try: node=dir.entries[norm_name]
                except KeyError: node=dir.dir_on_disk(self.name)
                if node and node.exists() and \
                    (isinstance(dir, Dir) or isinstance(dir, Entry)):
                        return node
        return self

    def sconsign(self):
        if not self._sconsign:
            import SCons.SConsign
            self._sconsign=SCons.SConsign.ForDirectory(self)
        return self._sconsign

    def srcnode(self):
        if self.srcdir:
            return self.srcdir
        return Base.srcnode(self)

    def get_timestamp(self):
        stamp=0
        for kid in self.children():
            if kid.get_timestamp() > stamp:
                stamp=kid.get_timestamp()
        return stamp

    def entry_abspath(self, name):
        return self.abspath + os.sep + name

    def entry_path(self, name):
        return self.path + os.sep + name

    def entry_tpath(self, name):
        return self.tpath + os.sep + name

    def must_be_a_Dir(self):
        return self

    def entry_exists_on_disk(self, name):
        """__cacheable__"""
        return self.fs.exists(self.entry_abspath(name))

    def srcdir_list(self):
        """__cacheable__"""
        result=[]

        dirname='.'
        dir=self
        while dir:
            if dir.srcdir:
                d=dir.srcdir.Dir(dirname)
                if d.is_under(dir): break
                result.append(d)
            dirname=dir.name + os.sep + dirname
            dir=dir.up()
        return result
    def srcdir_duplicate(self, name):
        for dir in self.srcdir_list():
            if dir.entry_exists_on_disk(name):
                srcnode=dir.File(name)
                if self.duplicate:
                    node=self.File(name)
                    node.do_duplicate(srcnode)
                    return node
                else: return srcnode
        return None

    def srcdir_find_file(self, filename):
        """__cacheable__"""
        def func(node):
            if (isinstance(node, File) or isinstance(node, Entry)) and \
               (node.is_derived() or node.is_pseudo_derived() or node.exists()):
                    return node
            return None
        norm_name=_my_normcase(filename)
        for rdir in self.get_all_rdirs():
            try: node=rdir.entries[norm_name]
            except KeyError: node=rdir.file_on_disk(filename)
            else: node=func(node)
            if node: return node, self
        for srcdir in self.srcdir_list():
            for rdir in srcdir.get_all_rdirs():
                try: node=rdir.entries[norm_name]
                except KeyError: node=rdir.file_on_disk(filename)
                else: node=func(node)
                if node: return File(filename, self, self.fs), srcdir
        return None, None
    def dir_on_disk(self, name):
        if self.entry_exists_on_disk(name):
            try: return self.Dir(name)
            except TypeError: pass
        return None
    def file_on_disk(self, name):
        if self.entry_exists_on_disk(name):
            try: return self.File(name)
            except TypeError: pass
        return self.srcdir_duplicate(name)
class RootDir(Dir):
    def __init__(self, name, fs):
        if __debug__: logInstanceCreation(self, 'Node.FS.RootDir')
        self.abspath=''
        self.path=''
        self.tpath=''
        self.path_elements=[]
        self.duplicate=0
        Base.__init__(self, name, self, fs)
        self.abspath=name + os.sep
        self.path=name + os.sep
        self.tpath=name + os.sep
        self._morph()
    def __str__(self):
        return self.abspath
    def entry_abspath(self, name):
        return self.abspath + name
    def entry_path(self, name):
        return self.path + name
    def entry_tpath(self, name):
        return self.tpath + name
    def is_under(self, dir):
        if self is dir: return 1
        else: return 0
    def up(self):
        return None
    def get_dir(self):
        return None
    def src_builder(self):
        return _null
class NodeInfo(SCons.Node.NodeInfo):
    bsig=None
    def __cmp__(self, other):
        return cmp(self.bsig, other.bsig)
    def update(self, node):
        self.timestamp=node.get_timestamp()
        self.size=node.getsize()
class BuildInfo(SCons.Node.BuildInfo):
    def __init__(self, node):
        SCons.Node.BuildInfo.__init__(self, node)
        self.node=node
    def convert_to_sconsign(self):
        rel_path=self.node.rel_path
        delattr(self, 'node')
        for attr in ['bsources', 'bdepends', 'bimplicit']:
            try:
                val=getattr(self, attr)
            except AttributeError:
                pass
            else: setattr(self, attr, map(rel_path, val))
    def convert_from_sconsign(self, dir, name):
        self.node=dir.Entry(name)
        Entry_func=self.node.dir.Entry
        for attr in ['bsources', 'bdepends', 'bimplicit']:
            try:
                val=getattr(self, attr)
            except AttributeError:
                pass
            else:
                setattr(self, attr, map(Entry_func, val))

class File(Base):
    def diskcheck_match(self):
        diskcheck_match(self, self.fs.isdir, "Directory %s found where file expected.")
    def __init__(self, name, directory, fs):
        if __debug__: logInstanceCreation(self, 'Node.FS.File')
        Base.__init__(self, name, directory, fs)
        self._morph()
    def Entry(self, name):
        return self.fs.Entry(name, self.cwd)
    def Dir(self, name):
        return self.fs.Dir(name, self.cwd)
    def Dirs(self, pathlist):
        return map(lambda p, s=self: s.Dir(p), pathlist)
    def File(self, name):
        return self.fs.File(name, self.cwd)
    def RDirs(self, pathlist):
        return self.fs.Rfindalldirs(pathlist, self.cwd)
    def _morph(self):
        """__cache_reset__"""
        self.scanner_paths={}
        if not hasattr(self, '_local'): self._local=0
    def disambiguate(self):
        return self
    def scanner_key(self):
        return self.get_suffix()
    def get_contents(self):
        if not self.rexists(): return ''
        return open(self.rfile().abspath, "rb").read()
    def get_timestamp(self):
        if self.rexists(): return self.rfile().getmtime()
        else: return 0
    def store_info(self, obj):
        entry=self.get_stored_info()
        entry.merge(obj)
        self.dir.sconsign().set_entry(self.name, entry)
    def get_stored_info(self):
        "__cacheable__"
        try:
            stored=self.dir.sconsign().get_entry(self.name)
        except (KeyError, OSError):
            return self.new_binfo()
        else:
            if not hasattr(stored, 'ninfo'):
                ninfo=stored.ninfo=self.new_ninfo()
                for attr in ninfo.__dict__.keys():
                    try:
                        setattr(ninfo, attr, getattr(stored, attr))
                    except AttributeError:
                        pass
            return stored

    def get_stored_implicit(self):
        binfo=self.get_stored_info()
        try: return binfo.bimplicit
        except AttributeError: return None

    def rel_path(self, other):
        return self.dir.rel_path(other)

    def get_found_includes(self, env, scanner, path):
        """__cacheable__"""
        if not scanner: return []
        return scanner(self, env, path)

    def _createDir(self):
        self.dir._create()

    def retrieve_from_cache(self):
        b=self.is_derived()
        if not b and not self.has_src_builder():
            return None
        if b and self.fs.CachePath:
            if self.fs.cache_show:
                if CacheRetrieveSilent(self, [], None, execute=1) ==0:
                    self.build(presub=0, execute=0)
                    return 1
            elif CacheRetrieve(self, [], None, execute=1) ==0:
                return 1
        return None

    def built(self):
        """__cache_reset__"""
        if self.fs.CachePath and self.exists(): CachePush(self, [], None)
        self.fs.clear_cache()
        SCons.Node.Node.built(self)
    def visited(self):
        if self.fs.CachePath and self.fs.cache_force and self.exists(): CachePush(self, None, None)
    def has_src_builder(self):
        try:
            scb=self.sbuilder
        except AttributeError:
            if self.rexists():
                scb=None
            else:
                scb=self.dir.src_builder()
                if scb is _null: scb=None
                if scb is not None:
                    self.builder_set(scb)
            self.sbuilder=scb
        return not scb is None

    def alter_targets(self):
        if self.is_derived():
            return [], None
        return self.fs.build_dir_target_climb(self, self.dir, [self.name])

    def is_pseudo_derived(self):
        "__cacheable__"
        return self.has_src_builder()

    def _rmv_existing(self):
        '__cache_reset__'
        Unlink(self, [], None)
        
    def prepare(self):
        SCons.Node.Node.prepare(self)

        if self.get_state() !=SCons.Node.up_to_date:
            if self.exists():
                if self.is_derived() and not self.precious:
                    self._rmv_existing()
            else:
                try:
                    self._createDir()
                except SCons.Errors.StopError, drive:
                    desc="No drive `%s' for target `%s'." % (drive, self)
                    raise SCons.Errors.StopError, desc
    def remove(self):
        if self.exists() or self.islink():
            self.fs.unlink(self.path)
            return 1
        return None
    def do_duplicate(self, src):
        self._createDir()
        try:
            Unlink(self, None, None)
        except SCons.Errors.BuildError:
            pass
        try:
            Link(self, src, None)
        except SCons.Errors.BuildError, e:
            desc="Cannot duplicate `%s' in `%s': %s." % (src.path, self.dir.path, e.errstr)
            raise SCons.Errors.StopError, desc
        self.linked=1
        self.clear()

    def exists(self):
        "__cacheable__"
        if self.duplicate and not self.is_derived() and not self.linked:
            src=self.srcnode()
            if src is self:
                return Base.exists(self)
            src=src.rfile()
            if src.abspath !=self.abspath and src.exists():
                self.do_duplicate(src)
        return Base.exists(self)

    def new_binfo(self):
        return BuildInfo(self)

    def new_ninfo(self):
        ninfo=NodeInfo()
        ninfo.update(self)
        return ninfo

    def get_csig(self, calc=None):
        try:
            return self.binfo.ninfo.csig
        except AttributeError:
            pass

        if calc is None:
            calc=self.calculator()

        max_drift=self.fs.max_drift
        mtime=self.get_timestamp()
        use_stored=max_drift >=0 and (time.time() - mtime) > max_drift

        csig=None
        if use_stored:
            old=self.get_stored_info().ninfo
            try:
                if old.timestamp and old.csig and old.timestamp ==mtime:
                    csig=old.csig
            except AttributeError:
                pass
        if csig is None:
            csig=calc.module.signature(self)

        binfo=self.get_binfo()
        ninfo=binfo.ninfo
        ninfo.csig=csig
        ninfo.update(self)

        if use_stored:
            self.store_info(binfo)

        return csig
    def current(self, calc=None):
        self.binfo=self.gen_binfo(calc)
        return self._cur2()
    def _cur2(self):
        "__cacheable__"
        if self.always_build:
            return None
        if not self.exists():
            r=self.rfile()
            if r !=self:
                old=r.get_stored_info()
                new=self.get_binfo()
                if new ==old:
                    if self._local:
                        LocalCopy(self, r, None)
                        self.store_info(new)
                    return 1
            return None
        else:
            old=self.get_stored_info()
            new=self.get_binfo()
            return (new ==old)

    def rfile(self):
        "__cacheable__"
        if not self.exists():
            norm_name=_my_normcase(self.name)
            for dir in self.dir.get_all_rdirs():
                try: node=dir.entries[norm_name]
                except KeyError: node=dir.file_on_disk(self.name)
                if node and node.exists() and \
                   (isinstance(node, File) or isinstance(node, Entry) \
                    or not node.is_derived()):
                        return node
        return self

    def rstr(self):
        return str(self.rfile())

    def cachepath(self):
        if not self.fs.CachePath:
            return None, None
        ninfo=self.get_binfo().ninfo
        if not hasattr(ninfo, 'bsig'):
            raise SCons.Errors.InternalError, "cachepath(%s) found no bsig" % self.path
        elif ninfo.bsig is None:
            raise SCons.Errors.InternalError, "cachepath(%s) found a bsig of None" % self.path
        cache_sig=SCons.Sig.MD5.collect([ninfo.bsig, self.path])
        subdir=string.upper(cache_sig[0])
        dir=os.path.join(self.fs.CachePath, subdir)
        return dir, os.path.join(dir, cache_sig)

    def target_from_source(self, prefix, suffix, splitext=SCons.Util.splitext):
        return self.dir.File(prefix + splitext(self.name)[0] + suffix)

    def must_be_a_Dir(self):
        raise TypeError, "Tried to lookup File '%s' as a Dir." % self.path

default_fs=None

def find_file(filename, paths, verbose=None):
    """__cacheable__"""
    if verbose:
        if not SCons.Util.is_String(verbose):
            verbose="find_file"
        if not callable(verbose):
            verbose='  %s: ' % verbose
            verbose=lambda s, v=verbose: sys.stdout.write(v + s)
    else: verbose=lambda x: x
    if callable(paths): paths=paths()
    paths=map(lambda p: p.must_be_a_Dir(), paths)
    filedir, filename=os.path.split(filename)
    if filedir:
        def filedir_lookup(p, fd=filedir):
            try:
                return p.Dir(fd)
            except TypeError:
                return None
        paths=filter(None, map(filedir_lookup, paths))
    for dir in paths:
        verbose("looking for '%s' in '%s' ...\n" % (filename, dir))
        node, d=dir.srcdir_find_file(filename)
        if node:
            verbose("... FOUND '%s' in '%s'\n" % (filename, d))
            return node
    return None

def find_files(filenames, paths):
    nodes=map(lambda x, paths=paths: find_file(x, paths), filenames)
    return filter(None, nodes)
